#include <algorithm>
#include "unittest.h"
#include "maze.h"
#include "point.h"

#include <sstream>

using namespace std;

UnitTest(testMazeConstructor)
{
    Maze m;
    assertThat(m.width(), isEqualTo(0));
    assertThat(m.height(), isEqualTo(0));
    assertThat(m(0, 0), isEqualTo(Maze::Wall));
}

UnitTest(testMazeInput)
{
    Maze m;
    istringstream input(
        string("4 5\n") +
        "####\n" +
        "#s #\n" +
        "#  #\n" +
        "# g#\n" +
        "####\n");
    input >> m;
    assertThat(m.width(), isEqualTo(4));
    assertThat(m.height(), isEqualTo(5));
    assertThat(m(0, 0), isEqualTo(Maze::Wall));
    assertThat(m(1, 1), isEqualTo(Maze::Start));
    assertThat(m(2, 3), isEqualTo(Maze::Goal));

    assertThat(m(2, 2), isEqualTo(' '));
    m.set(2, 2, Maze::Wall);
    assertThat(m(2, 2), isEqualTo(Maze::Wall));

    assertThat(m(10, 25), isEqualTo(Maze::Wall));

    assertThat(m.goalPosition(), isEqualTo(Point({2,3})));
    assertThat(m.startPosition(), isEqualTo(Point({1,1})));
    
}

UnitTest(testMazeOutput)
{
    Maze m;
    string maze =
        string("####\n") +
        "#s #\n" +
        "#  #\n" +
        "# g#\n" +
        "####\n";

    istringstream input(string("4 5\n") + maze);

    input >> m;

    ostringstream output;
    output << m;
    string outstring = output.str();
    auto pos = remove(outstring.begin(), outstring.end(), '\r');
    outstring.erase(pos, outstring.end());
    assertThat(outstring, isEqualTo(maze));
}